//
//  ImageViewController.swift
//  ImageViewer
//
//  Created by Stephen Freund on 10/15/18.
//  Copyright © 2018 Stephen Freund. All rights reserved.
//

import UIKit

/**
 A self-contained controller to display an image loaded from a URL.
 It demonstrates the use of a scroll view and delegation.
 */

class ImageViewController: UIViewController, UIScrollViewDelegate {
  
  // - MARK: The model!
  
  public var imageURL : URL? {
    didSet {
      image = nil
      fetchImage()
    }
  }
  
  // - MARK: The view!
  
  // We'll create a view programmatically this time, rather
  // than in the storyboard.
  private var imageView = UIImageView()
  
  // Always set up the necessary properties of a scroll view outlet
  // when it is created.
  @IBOutlet weak var scrollView: UIScrollView! {
    didSet {
      scrollView.delegate = self
      scrollView.contentSize = imageView.frame.size
      scrollView.addSubview(imageView)
      scrollView.maximumZoomScale = 2.0
      scrollView.minimumZoomScale = 0.03
    }
  }
  
  // - MARK: UIScrollView delegate methods
  
  // -Returns: the view we wish to scroll around in.  In this case,
  // that's our imageView.
  func viewForZooming(in scrollView: UIScrollView) -> UIView? {
    return imageView
  }
  
  // - MARK: Our private implementation
  
  /**
   A compute property to assist in writing the rest of the controller, since
   whenever we wish to change the image, we need to do several tasks.
   */
  private var image : UIImage? {
    set {
      // Be careful here: If image is assigned to in preparation
      // of segue, our scrollView outlet will not have
      // been set yet.
      
      // Added after lecture -- reset zoom/offset before changing image:
      scrollView?.zoomScale = 1
      scrollView?.contentOffset = CGPoint.zero
      
      imageView.image = newValue
      imageView.sizeToFit()
      scrollView?.contentSize = imageView.frame.size
    }
    get {
      return imageView.image
    }
  }
  
  /**
   Load the data from a URL and convert it into the image
   we wish to display.
   */
  private func fetchImage() {
    if let url = imageURL,
      let urlContents = try? Data(contentsOf: url) {
      image = UIImage(data: urlContents)
    }
  }
  
  // Just load a default image for to start with.
  override func viewDidLoad() {
    super.viewDidLoad()
    imageURL = URL(string: "http://timedotcom.files.wordpress.com/2017/06/williams-college-senate-free-speech.jpg")
  }
}

